:: =============================================================================
:: =        Copyright (C) 2016-2016 Noel Carboni, ProDigital Software.         =
:: =                            All Rights Reserved.                           =
:: =                                                                           =
:: =  You may use this software freely and share it with others as long as     =
:: =       keep this notice and the documentation listed herein intact.        =
:: =============================================================================
::
::+=============================================================================
::
:: Name:        CortanaAndAppsRemoval
:: Ident:       1.2
:: Type:        User Command Task
:: Language:    MS-DOS Batch
:: Index:       Facilitates removal of Cortana and other Apps
::
::
:: Function:
::
::      This script facilitates the permanent removal of Cortana and other Apps either
::      initially or after an upgrade.
::
::      USE AT YOUR OWN RISK!
::
::      It PERMANENTLY removes Apps, though they could probably be restored through 
::      re-application of the Windows build.  I've never actually tried that, because
::      I can't imagine wanting Apps.
::
::
:: Synopsis:
::
::      CortanaAndAppsRemoval
::
::
:: Additional Information:
::
::      MAKE BACKUPS and use this script at your own risk!
::
::      An attempt is made to store user choices for presentation as defaults on the
::      next run.  It is imagined that this script will lie dormant on the machine,
::      along with the "saved answers" file, until a new "in-place upgrade" is
::      released by Microsoft.  At that time the user runs this script and is prompted 
::      again to make important decisions about each tweak / re-tweak, and his/her
::      prior choices are presented as defaults.
::
::      A minimal set of tools is expected to be available via the current directory
::      or the path, including:
::
::          ShowTime.exe and Repeat.exe from Noel Carboni	http://Noel.ProDigitalSoftware.com/files/Tools.zip
::          sleep.exe and tail.exe from GnuWin32 Toolkit	http://gnuwin32.sourceforge.net/packages/coreutils.htm
::          which.exe from GnuWin32 Toolkit					http://gnuwin32.sourceforge.net/packages/which.htm
::          grep.exe from GnuWin32 Toolkit					http://gnuwin32.sourceforge.net/packages/grep.htm
::          sed.exe from GnuWin32 Toolkit					http://gnuwin32.sourceforge.net/packages/sed.htm
::          SQLite3.exe from								https://www.sqlite.org/download.html
::          SetACL.exe from Helge Klein						https://helgeklein.com/download/
::
::
:: Maintenance Information:
::
::      Modification History:
::      ---------------------
::
::      February 3, 2016    Noel Carboni
::
::      Initial conception as a batch file.  It may grow up to be a full-fledged
::      application one day...
::
::      If you find this script helpful, please feel free to send donations
::      my way at:  NCarboni@ProDigitalSoftware.com
::
::-=============================================================================
::
::  Start of process.
::
	@ECHO OFF
::
::  Try to change the current directly to the one containing this file.  This won't be
::  needed on test systems with the various tools already in the path.
::
	CD /D "%~dp0" >nul 2>&1
::
    ECHO.
    ECHO ----------------------------------------------------------------------------------------
    ECHO.
::
    which grep >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which sed >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which sleep >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which tail >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which REPEAT >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which SQLite3 >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
    which SetACL >nul
    IF ERRORLEVEL 1 GOTO UTILS_FAIL
::
:: Success if we fall through to here.
::
    GOTO INITIAL_PROMPT
::
:UTILS_FAIL
    ECHO.
    ECHO ERROR:  One or more tools needed by this script cannot be located.
    GOTO END_OF_PROCEDURE
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:INITIAL_PROMPT
::
    ECHO This script will allow you to permanently remove Cortana and most other Apps. 
    ECHO Your previous answers, if any, are shown as defaults.
    ECHO.
    ECHO                                  WARNING!  
    ECHO.
    ECHO This is a disruptive process!  Make backups, save your data and close all other 
    ECHO applications before continuing.  You will want to reboot after running this script.
    ECHO.
    SET /P _WRT_ANSWER="Continue (Y/N)? "
    IF "%_WRT_ANSWER%" == "Y" GOTO START_RUN
    IF "%_WRT_ANSWER%" == "y" GOTO START_RUN
::
    GOTO QUIT_PROCEDURE
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:START_RUN
::
    SET _WRT_Info=C:\ProgramData\CortanaAndAppsRemoval
    IF NOT EXIST "%_WRT_Info%" MD "%_WRT_Info%" >nul 2>&1
::
    IF EXIST "%_WRT_Info%\SavedAnswers.bat" (
        CALL "%_WRT_Info%\SavedAnswers.bat"
        ECHO.
        GREP -F "CortanaAndAppsRemoval was last run" "%_WRT_Info%\SavedAnswers.bat" | TAIL -1
    )
::
    ECHO :: CortanaAndAppsRemoval was last run at %TIME% on %DATE% ::>>"%_WRT_Info%\SavedAnswers.bat"
::
::=======================================================================================================================================================================================
::
    ECHO.
    ECHO ----------------------------------------------------------------------------------------
    ECHO.
    ECHO                                   Remove Cortana
    ECHO.
    ECHO Microsoft pre-installs a particularly irritating and intrusive App called Cortana.  
    ECHO You see it in your Task Manager as SearchUI.  Cortana CAN be removed, which will
    ECHO provide you benefits in resource usage and privacy.
    ECHO.
::
    IF "%_WRT_REMOVE_CORTANA%" == "" SET _WRT_REMOVE_CORTANA=Y
    SET _WRT_ANSWER=%_WRT_REMOVE_CORTANA%
    SET /P _WRT_ANSWER="Remove Cortana (Y/N/Q, default: %_WRT_ANSWER%)? "
::
    IF "%_WRT_ANSWER%" == "Q" GOTO QUIT_PROCEDURE
    IF "%_WRT_ANSWER%" == "q" GOTO QUIT_PROCEDURE
::
    IF "%_WRT_ANSWER%" == "" SET _WRT_ANSWER=%_WRT_REMOVE_CORTANA%
    IF NOT "%_WRT_ANSWER%" == "%_WRT_REMOVE_CORTANA%" ECHO SET _WRT_REMOVE_CORTANA=%_WRT_ANSWER%>>"%_WRT_Info%\SavedAnswers.bat"
    SET _WRT_REMOVE_CORTANA=
::
    IF "%_WRT_ANSWER%" == "Y" GOTO DO_REMOVE_CORTANA
    IF "%_WRT_ANSWER%" == "y" GOTO DO_REMOVE_CORTANA
::
    ECHO.
    ECHO Skipping Removal of Cortana.
    GOTO END_REMOVE_CORTANA
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:DO_REMOVE_CORTANA
::
    ECHO.
    ECHO Removing Cortana.
::
::  Get lists of the package names to guide our removal operations.
::
    PowerShell "Get-AppXPackage -AllUsers | Select PackageFullName" >_WRT_AllPackages.txt
    grep -F "Microsoft.Windows.Cortana" _WRT_AllPackages.txt >_WRT_Cortana.txt
    IF NOT ERRORLEVEL 1 GOTO FOUND_CORTANA
::
    ECHO.
    ECHO ERROR:  Cortana not found - possibly already removed?
    GOTO END_REMOVE_CORTANA
::
:FOUND_CORTANA
::
::  Get the package short name (e.g., Microsoft.Windows.Cortana)
::
    sed -r -e "s/[ \t]+//g" -e "s/^/SET _WRT_CORTANA=/" _WRT_Cortana.txt >_WRT_Cortana.bat
    CALL _WRT_Cortana.bat
::
::  Make the StateRepository-Machine database writable by mere mortals.
::
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository" -actn setowner -ownr "n:Administrators"
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository" -actn ace -ace "n:Administrators;p:full"
::
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" -actn setowner -ownr "n:Administrators"
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" -actn ace -ace "n:Administrators;p:full"
::
::  Now we must change the "IsInBox" value to 0 for all existing entries in the Package table.
::
    SQLite3 "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" "UPDATE Package SET IsInBox=0 WHERE IsInBox=1 AND PackageFullName LIKE '%%Cortana%%';"
    IF NOT ERRORLEVEL 1 GOTO DATABASE_CHANGED_OK
::
    ECHO.
    ECHO ERROR:  Could not alter the StateRepository-Machine.srd database - App removal failed.
    GOTO END_REMOVE_CORTANA
::
:DATABASE_CHANGED_OK
::
::  A registry entry has to be removed or the system will try to bring back Cortana.
::
::  We need to find the package identifier assigned to Cortana for this particular Windows distribution.
::
    REG EXPORT "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Appx\AppxAllUserStore\InboxApplications" _WRT_InBox.tmp /y >nul
    TYPE _WRT_InBox.tmp | grep -F "InboxApplications\Microsoft.Windows.Cortana" | sed -r -e "s/^\[//" -e "s/\]//" -e "s/[ \t]+//g" -e "s/^/SET _WRT_CORTANAPKG=/" >_WRT_Cortana.bat
    CALL _WRT_Cortana.bat
::
    SetACL -silent -ot "reg" -on "%_WRT_CORTANAPKG%" -actn setowner -ownr "n:Administrators"
    SetACL -silent -ot "reg" -on "%_WRT_CORTANAPKG%" -actn ace -ace "n:Administrators;p:full" 
    REG DELETE           "%_WRT_CORTANAPKG%" /f >nul
::
::  Now remove the Cortana App
::
    PowerShell "Remove-AppXPackage %_WRT_CORTANA%"
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
    ECHO.
    ECHO Removal of Cortana completed.
::
:END_REMOVE_CORTANA
::
    SET _WRT_CORTANA=
::
    DEL _WRT_Cortana.txt _WRT_Cortana.bat _WRT_AllPackages.txt _WRT_InBox.tmp >nul 2>&1
::
::=======================================================================================================================================================================================
::
    ECHO.
    ECHO ----------------------------------------------------------------------------------------
    ECHO.
    ECHO                               Remove Pre-Installed Apps
    ECHO.
    ECHO Microsoft pre-installs many Metro/Modern/Universal Apps.  They think XAML Apps represent
    ECHO the future of computing.  Trouble is, they're not worth much, AND they don't run at all
    ECHO with UAC disabled.  Windows 10 can run just fine with just the following remaining 
    ECHO Apps, which are NOT removed by this part of the procedure:
    ECHO.
    ECHO   Microsoft.Windows.ShellExperienceHost - Necessary to run any App
    ECHO   windows.immersivecontrolpanel -         Necessary to configure Windows
    ECHO.
::
    IF "%_WRT_REMOVE_APPS%" == "" SET _WRT_REMOVE_APPS=Y
    SET _WRT_ANSWER=%_WRT_REMOVE_APPS%
    SET /P _WRT_ANSWER="Remove most pre-installed Apps (Y/N/Q, default: %_WRT_ANSWER%)? "
::
    IF "%_WRT_ANSWER%" == "Q" GOTO QUIT_PROCEDURE
    IF "%_WRT_ANSWER%" == "q" GOTO QUIT_PROCEDURE
::
    IF "%_WRT_ANSWER%" == "" SET _WRT_ANSWER=%_WRT_REMOVE_APPS%
    IF NOT "%_WRT_ANSWER%" == "%_WRT_REMOVE_APPS%" ECHO SET _WRT_REMOVE_APPS=%_WRT_ANSWER%>>"%_WRT_Info%\SavedAnswers.bat"
    SET _WRT_REMOVE_APPS=
::
    IF "%_WRT_ANSWER%" == "Y" GOTO DO_REMOVE_APPS
    IF "%_WRT_ANSWER%" == "y" GOTO DO_REMOVE_APPS
::
    ECHO.
    ECHO Skipping Removal of Pre-installed Apps.
    GOTO END_REMOVE_APPS
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:DO_REMOVE_APPS
::
::  First, remove all the provisioned App packages
::
    ECHO.
    ECHO Removing all provisioned packages.  This may take some time.
::
    PowerShell "Get-AppXProvisionedPackage -Online | Remove-AppXProvisionedPackage -Online"
::
::
::  Make the StateRepository-Machine database writable by mere mortals.
::
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository" -actn setowner -ownr "n:Administrators"
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository" -actn ace -ace "n:Administrators;p:full"
::
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" -actn setowner -ownr "n:Administrators"
    SetACL -silent -ot "file" -on "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" -actn ace -ace "n:Administrators;p:full"
::
::  Now we must change the "IsInBox" value to 0 for all existing entries in the Package table.
::
    SQLite3 "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" "UPDATE Package SET IsInBox=0 WHERE IsInBox=1;"
    IF NOT ERRORLEVEL 1 GOTO ENUMERATE_PACKAGES
::
    ECHO.
    ECHO Could not alter the StateRepository-Machine.srd database - App removal failed.
    GOTO END_REMOVE_APPS
::
:ENUMERATE_PACKAGES
::
::  Get lists of the package names to guide our removal operations.
::
    PowerShell "Get-AppXPackage -AllUsers | Select PackageFullName" >_WRT_AllPackages.txt
::
::
::  Now remove all the Apps except ShellExperienceHost and immersivecontrolpanel and run-time packages
::
    ECHO.
    ECHO Removing installed packages.  This may take some time.
    ECHO.
::
    sed -r -e "/Microsoft\.NET\./d" -e "/Microsoft\.VCLibs\./d" -e "/Cortana/d" -e "/PackageFullName/d" -e "/---------------/d" -e "/^Microsoft\.Windows\.ShellExperienceHost/d" -e "/^windows\.immersivecontrolpanel/d" -e "/^[ \t]+$/d" -e "/^$/d" _WRT_AllPackages.txt >_WRT_TrimmedPackages.txt
    REPEAT "PowerShell Remove-AppXPackage %%s" @_WRT_TrimmedPackages.txt >_WRT_RemovePackages.bat
    CALL _WRT_RemovePackages.bat
::
::  Now that the apps are all gone, remove the framework and run-time packages
::
::  Cortana automatically reappears with the IsInBox setting of 1.  Kill it again.
::
    SQLite3 "%ProgramData%\Microsoft\Windows\AppRepository\StateRepository-Machine.srd" "UPDATE Package SET IsInBox=0 WHERE IsInBox=1;"
::
    PowerShell "Get-AppXPackage -AllUsers | Select PackageFullName" >_WRT_RemainingPackages.txt
    sed -r -e "/Cortana/d" -e "/PackageFullName/d" -e "/---------------/d" -e "/^Microsoft\.Windows\.ShellExperienceHost/d" -e "/^windows\.immersivecontrolpanel/d" -e "/^[ \t]+$/d" -e "/^$/d" _WRT_RemainingPackages.txt >_WRT_TrimmedPackages.txt
    REPEAT "PowerShell Remove-AppXPackage %%s" @_WRT_TrimmedPackages.txt >_WRT_RemovePackages.bat
    CALL _WRT_RemovePackages.bat
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
    ECHO.
    ECHO Removal of Pre-installed Apps Completed.
::
:END_REMOVE_APPS
::
    DEL _WRT_AllPackages.txt _WRT_TrimmedPackages.txt _WRT_RemainingPackages.txt _WRT_RemovePackages.bat >nul 2>&1
::
::=======================================================================================================================================================================================
::
:QUIT_PROCEDURE
::
    ECHO.
    ECHO ----------------------------------------------------------------------------------------
    ECHO.
::
::  Show the user what's left.
::
    ECHO Your remaining installed Apps:
	ECHO ------------------------------
    ECHO.
	ECHO Get-AppXPackage -AllUsers ^| Select Name, PackageFullName
    PowerShell "Get-AppXPackage -AllUsers | Select Name, PackageFullName"
::
    ECHO Your remaining proisioned Apps:
	ECHO -------------------------------
    ECHO.
	ECHO Get-AppXProvisionedPackage -Online ^| Select PackageName
    PowerShell "Get-AppXProvisionedPackage -Online | Select PackageName"
::
    ECHO.
    ECHO ----------------------------------------------------------------------------------------
    ECHO.
    ECHO Exiting CortanaAndAppsRemoval - remember to reboot to take advantage of any changes.
    ECHO.
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:CLEAN_UP
::
    SET _WRT_ANSWER=
    SET _WRT_Info=
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:END_OF_PROCEDURE
::
::==============================================================================
::
::  End of file.
::
::==============================================================================
